# Generated by Django 2.2.8 on 2019-12-10 17:20
from django.contrib.postgres.operations import UnaccentExtension
from django.db import migrations


class Migration(migrations.Migration):

    operations = [UnaccentExtension()]
