# Generated by Django 3.1.13 on 2021-11-06 12:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("superadmin", "0001_action_menu"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="menu",
            options={"ordering": ("sequence",)},
        ),
    ]
